import json
import logging
from abc import ABC, abstractmethod
from typing import Literal

from aiokafka import AIOKafkaConsumer

logger = logging.getLogger(__name__)


class AbstractKafkaConsumer(ABC):
    @abstractmethod
    async def start(self) -> None: ...

    @abstractmethod
    async def stop(self) -> None: ...

    @abstractmethod
    async def consume(self) -> None: ...

    @abstractmethod
    async def process_message(self, message: dict) -> None: ...


class KafkaConsumer(AbstractKafkaConsumer):
    def __init__(
        self,
        topic_name: str,
        bootstrap_servers: str,
        group_id: str,
        auto_offset_reset: Literal["latest", "earliest"] = "latest",
    ):
        self.topic_name = topic_name
        self.bootstrap_servers = bootstrap_servers
        self.group_id = group_id
        self.consumer = None
        self.auto_offset_reset = auto_offset_reset

    async def start(self):
        self.consumer = AIOKafkaConsumer(
            self.topic_name,
            bootstrap_servers=self.bootstrap_servers,
            group_id=self.group_id,
            auto_offset_reset=self.auto_offset_reset,
        )
        await self.consumer.start()

        logger.info(f"{self.__class__.__name__}: Consumer started")

    async def stop(self):
        if self.consumer:
            await self.consumer.stop()
            logger.info(f"{self.__class__.__name__}: Consumer stopped")

    async def consume(self):
        try:
            async for msg in self.consumer:
                message = json.loads(msg.value.decode("utf-8"))
                await self.process_message(message)
        except Exception as e:
            logger.error(f"{self.__class__.__name__}: Error consuming message: {e}")
