import json
import logging
from abc import ABC, abstractmethod

from aiokafka import AIOKafkaProducer
from aiokafka.admin import AIOKafkaAdminClient, NewTopic

logger = logging.getLogger(__name__)


class AbstractKafkaProducer(ABC):
    @abstractmethod
    async def start(self) -> None: ...

    @abstractmethod
    async def stop(self) -> None: ...

    @abstractmethod
    async def send(self, topic: str, payload: dict) -> None: ...


class KafkaProducer(AbstractKafkaProducer):
    def __init__(self, bootstrap_servers: str, partitions: int = 3):
        self.bootstrap_servers = bootstrap_servers
        self.partitions = partitions
        self.producer = None

    async def start(self):
        if self.producer is None or self.producer._closed:
            self.producer = AIOKafkaProducer(bootstrap_servers=self.bootstrap_servers)
            await self.producer.start()
            logger.info(f"{self.__class__.__name__}: Producer started")

    async def stop(self):
        if self.producer:
            await self.producer.stop()
            logger.info(f"{self.__class__.__name__}: Producer stopped")

    async def send(self, topic: str, payload: dict):
        await self._create_topic_if_missing(topic)
        await self.producer.send_and_wait(
            topic,
            value=json.dumps(payload, default=str).encode(),
        )

    async def _create_topic_if_missing(self, topic: str) -> None:
        admin_client = AIOKafkaAdminClient(bootstrap_servers=self.bootstrap_servers)
        await admin_client.start()
        try:
            topics = await admin_client.list_topics()
            if topic not in topics:
                topic = NewTopic(
                    name=topic,
                    num_partitions=self.partitions,
                    replication_factor=1,
                )
                await admin_client.create_topics(new_topics=[topic], validate_only=False)
                logger.info(f"{self.__class__.__name__}: Topic '{topic}' created.")
        except Exception as e:
            logger.error(f"{self.__class__.__name__}: Error creating topic: {e}")
        finally:
            await admin_client.close()
